@@ -165,7 +165,7 @@ class ApplicationPartPut(ApplicationBase):
              bool(rights.intersect(access.permissions, "Ww")),
              bool(rights.intersect(access.parent_permissions, "w")))
 
-        with self._storage.acquire_lock("w", user):
+        with self._storage.acquire_lock("w", user, path=path):
             item = next(iter(self._storage.discover(path)), None)
             parent_item = next(iter(
                 self._storage.discover(access.parent_path)), None)
@@ -282,6 +282,10 @@ class ApplicationPartPut(ApplicationBase):
                         logger.warning(
                             "Bad PUT request on %r (upload): %s", path, e, exc_info=True)
                         return httputils.BAD_REQUEST
+                if (item and item.uid == prepared_item.uid):
+                    logger.debug("PUT request updated existing item %r", path)
+                    headers = {"ETag": etag}
+                    return client.NO_CONTENT, headers, None
 
             headers = {"ETag": etag}
             return client.CREATED, headers, None
